import React, { useMemo, lazy, Suspense } from 'react';
import { ChevronRight, Search, BookOpen, Copy } from 'lucide-react';
import { rhymeSets } from './rhymeData';
import { Link, Route, Routes, Navigate, useLocation } from 'react-router-dom';
import { Layout } from './components/Layout';
import { SEO } from './components/SEO';
import { getBaseUrl, formatUrlPhrase } from './utils/urlUtils';
import { ErrorBoundary } from './components/ErrorBoundary';
import About from './pages/About';
import Contact from './pages/Contact';
import PrivacyPolicy from './pages/PrivacyPolicy';
import Terms from './pages/Terms';

// Lazy load RhymePage component
const RhymePage = lazy(() => import('./components/RhymePage'));

function getAllUniquePhrases() {
  return useMemo(() => {
    const phrases = new Set<string>();
    Object.keys(rhymeSets).forEach(phrase => {
      phrases.add(phrase);
      rhymeSets[phrase as keyof typeof rhymeSets].forEach(rhyme => {
        phrases.add(rhyme);
      });
    });
    return Array.from(phrases).sort();
  }, []);
}

function groupByFirstLetter(phrases: string[]) {
  return useMemo(() => phrases.reduce((acc: { [key: string]: string[] }, phrase) => {
    const firstLetter = phrase[0].toUpperCase();
    if (!acc[firstLetter]) {
      acc[firstLetter] = [];
    }
    acc[firstLetter].push(phrase);
    return acc;
  }, {}), [phrases]);
}

function RhymeList() {
  const allPhrases = getAllUniquePhrases();
  const groupedPhrases = groupByFirstLetter(allPhrases);
  const [searchTerm, setSearchTerm] = React.useState('');
  const [filteredPhrases, setFilteredPhrases] = React.useState<string[]>(allPhrases);
  const letters = Object.keys(groupedPhrases).sort();
  const totalRhymes = allPhrases.length;

  React.useEffect(() => {
    const filtered = allPhrases.filter(phrase =>
      phrase.toLowerCase().includes(searchTerm.toLowerCase())
    );
    setFilteredPhrases(filtered);
  }, [searchTerm, allPhrases]);

  const filteredGroupedPhrases = useMemo(() => {
    return filteredPhrases.reduce((acc: { [key: string]: string[] }, phrase) => {
      const firstLetter = phrase[0].toUpperCase();
      if (!acc[firstLetter]) {
        acc[firstLetter] = [];
      }
      acc[firstLetter].push(phrase);
      return acc;
    }, {});
  }, [filteredPhrases]);

  const filteredLetters = Object.keys(filteredGroupedPhrases).sort();

  return (
    <Layout>
      <SEO
        title="MultiRhyme - The Ultimate Multi-Syllable Rhyming Dictionary"
        description={`The ultimate rhyme dictionary for rap with ${totalRhymes}+ multi-syllable rhymes. Find rhyming words for hip-hop lyrics, rap songs, and poetry.`}
        keywords="rhyme dictionary"
        canonicalUrl={getBaseUrl()}
      />

      <div className="min-h-screen bg-gradient-dark">
        <div className="container mx-auto px-4 py-8">
          <div className="max-w-4xl mx-auto">
            <header className="text-center mb-12">
              <div className="mb-8">
                <h1 className="text-4xl md:text-5xl font-bold text-white mb-4" itemProp="headline">
                  MultiRhyme Dictionary: Find Multi-Syllable Rhymes for Rap
                </h1>
                <p className="text-xl text-gray-300 mb-2">
                  Browse <span className="text-purple-400">{totalRhymes}+</span> multi-syllable rhymes for hip-hop lyrics
                </p>
                <p className="text-base text-gray-400" itemProp="alternativeHeadline">
                  Perfect for rappers, hip-hop artists, songwriters, and poets
                </p>
              </div>

              <nav aria-label="Quick search" className="max-w-2xl mx-auto">
                <div className="relative">
                  <Search className="absolute left-4 top-1/2 transform -translate-y-1/2 text-gray-400 h-5 w-5" />
                  <input
                    type="text"
                    value={searchTerm}
                    onChange={(e) => setSearchTerm(e.target.value)}
                    placeholder="Search for rhymes..."
                    className="w-full pl-12 pr-4 py-4 bg-white/10 backdrop-blur-sm border border-white/20
                             rounded-lg text-white placeholder-gray-400 focus:outline-none focus:ring-2 
                             focus:ring-purple-500 focus:border-transparent transition-all duration-200
                             shadow-lg"
                    aria-label="Search rhymes"
                  />
                </div>
                {searchTerm && (
                  <p className="text-gray-400 mt-2 text-sm">
                    Found {filteredPhrases.length} matching rhymes
                  </p>
                )}
              </nav>
            </header>

            <section aria-labelledby="how-it-works" className="mb-12 card-gradient rounded-lg p-8">
              <h2 id="how-it-works" className="text-2xl md:text-3xl font-bold text-white mb-6 text-center">
                How MultiRhyme Works
              </h2>
              <div className="grid md:grid-cols-3 gap-8">
                <div className="text-center">
                  <div className="bg-purple-500/10 rounded-lg p-4 mb-4 inline-block">
                    <Search className="h-8 w-8 text-purple-400" />
                  </div>
                  <h3 className="text-xl font-semibold text-white mb-2">
                    <span className="text-purple-400">1.</span> Search Rhymes
                  </h3>
                  <p className="text-gray-300">
                    Enter any word or phrase to find multi-syllable rhymes that match your flow
                  </p>
                </div>
                <div className="text-center">
                  <div className="bg-purple-500/10 rounded-lg p-4 mb-4 inline-block">
                    <BookOpen className="h-8 w-8 text-purple-400" />
                  </div>
                  <h3 className="text-xl font-semibold text-white mb-2">
                    <span className="text-purple-400">2.</span> Browse Results
                  </h3>
                  <p className="text-gray-300">
                    Explore curated lists of multi-syllable rhymes organized by sound patterns
                  </p>
                </div>
                <div className="text-center">
                  <div className="bg-purple-500/10 rounded-lg p-4 mb-4 inline-block">
                    <Copy className="h-8 w-8 text-purple-400" />
                  </div>
                  <h3 className="text-xl font-semibold text-white mb-2">
                    <span className="text-purple-400">3.</span> Save & Share
                  </h3>
                  <p className="text-gray-300">
                    Copy your favorite rhymes or share them directly with other artists
                  </p>
                </div>
              </div>
            </section>

            <section aria-labelledby="rhyme-categories" className="card-gradient rounded-lg p-8">
              <h2 id="rhyme-categories" className="sr-only">Rhyme Categories</h2>
              <nav aria-label="Rhyme Categories">
                <div className={`grid grid-cols-2 md:grid-cols-3 lg:grid-cols-4 gap-4 ${filteredPhrases.length === 0 ? 'min-h-[200px]' : ''}`}>
                  {filteredPhrases.length === 0 && (
                    <div className="col-span-full text-center py-12">
                      <p className="text-gray-400 text-lg">
                        No rhymes found for "{searchTerm}". Try a different search term.
                      </p>
                    </div>
                  )}
                  {letters.map((letter) => (
                    <div key={letter} className="space-y-2">
                      <h3 className="text-xl font-bold text-purple-400 mb-2">{letter}</h3>
                      <ul className="space-y-2">
                        {filteredGroupedPhrases[letter]?.map((phrase) => (
                          <li key={phrase}>
                            <Link
                              to={`/rhymes/${encodeURIComponent(formatUrlPhrase(phrase))}/`}
                              className="flex items-center text-gray-300 hover:text-purple-400 transition-colors group focus:outline-none focus:ring-2 focus:ring-purple-500 rounded"
                              aria-label={`Find rhymes for ${phrase}`}
                            >
                              <ChevronRight className="h-4 w-4 mr-1 group-hover:translate-x-1 transition-transform" />
                              <span>{phrase}</span>
                            </Link>
                          </li>
                        ))}
                      </ul>
                    </div>
                  ))}
                </div>
              </nav>
            </section>
          </div>
        </div>
      </div>
    </Layout>
  );
}

function App() {
  const location = useLocation();

  return (
    <ErrorBoundary
      fallback={
        <div className="min-h-screen bg-gradient-dark flex items-center justify-center p-4">
          <div className="text-white text-center">
            <h2 className="text-xl mb-4">Something went wrong</h2>
            <p className="text-gray-400 mb-4">We're having trouble loading this page</p>
            <button 
              onClick={() => window.location.reload()}
              className="btn btn-secondary"
            >
              Try Again
            </button>
          </div>
        </div>
      }
    >
      <Routes>
        <Route path="/" element={<RhymeList />} />
        <Route path="/about" element={<About />} />
        <Route path="/contact" element={<Suspense fallback={<div>Loading...</div>}><Contact /></Suspense>} />
        <Route path="/privacy-policy" element={<PrivacyPolicy />} />
        <Route path="/terms" element={<Terms />} />
        <Route
          path="/rhymes/:phrase"
          element={
            <Suspense fallback={
              <div className="min-h-screen bg-gradient-dark flex items-center justify-center p-4">
                <div className="text-white text-xl text-center">
                  <div className="mb-4">Loading rhymes...</div>
                  <div className="text-sm text-gray-400">If this takes too long, try refreshing the page</div>
                </div>
              </div>
            }>
              <RhymePage />
            </Suspense>
          }
        />
        <Route path="*" element={<Navigate to="/" replace />} />
      </Routes>
    </ErrorBoundary>
  );
}

export default App;